/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        16.11.2008
 Description:    Webserver Config-File

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

/**
 * \ingroup main	Hauptprogramm
 *
 */

/**
 * \file
 * Konfigurationsdatei
 *
 * \author Ulrich Radig & W.Wallucks
 */

#ifndef _CONFIG_H_
	#define _CONFIG_H_	
	
	#define USE_ENC28J60	1	// NET_IO_BOARD (Pollin) mit Add-On Board / Webmodul (www.ulrichradig.de)
	
	//Konfiguration der PORTS (HEX)
	//1=OUTPUT / 0=INPUT
	#define OUTA 		0x00
	#define OUTC 		0x00
	#define OUTD 		0x00
	
	//Achtung!!!! an PORTB ist der ENC
	//nur ndern wenn man wei was man macht!


	//! Umrechnung von IP zu unsigned long
	#define IP(a,b,c,d) ((unsigned long)(d)<<24)+((unsigned long)(c)<<16)+((unsigned long)(b)<<8)+a

	//IP des Webservers
	#define MYIP		IP(192,168,178,26)	//!< default IP des Webservers

	//Netzwerkmaske
	#define NETMASK		IP(255,255,255,0)	//!< default Netzwerkmaske
	
	//IP des Routers
	#define ROUTER_IP	IP(192,168,178,1)		//!< default IP des Routers/Gateway
	
	//IP des DNS-Servers
	#define USE_DNS		1					//!< mit/ohne DNS-Client
	#define DNS_IP		IP(192,168,178,1)		//!< IP des DNS-Servers

	//NTP Einstellen der Zeit mittels NTP
	#define USE_NTP		1 					//!< 1 = NTP Client on
	#define NTP_IP		IP(85,10,196,184)	//!< IP des NTP-Servers z.B. Server 1.de.pool.ntp.org
	#define NTP_SERVER	"1.de.pool.ntp.org"	//!< hostname des NTP-Servers, wird ber DNS aufgelst

	// don't touch! -- NTP-Server wird ber DNS aufgelst
	#if USE_NTP
	#undef	USE_DNS
	#define USE_DNS			1
	#endif
	
	
	//! MAC Adresse des Webservers	
	#define MYMAC1	0x00
	#define MYMAC2	0x20
	#define MYMAC3	0x18
	#define MYMAC4	0xB1	
	#define MYMAC5	0x15
	#define MYMAC6	0x3F
	
	//Taktfrequenz
	//#define F_CPU 16000000UL	
	#define F_CPU 14745600UL
	//#define F_CPU 11059200UL
	
	#define TIMERBASE		25		//!< Zeitbasis fr Timerinterrupt ist 25 ms

	// Timertakt intern oder extern
	// dies hat nichts mit dem Prozessortakt zu tun!
	#define EXTCLOCK 0 //0=Intern 1=Externer Uhrenquarz


/** USART *************/
	#define BAUDRATE 	38400		//!< Baudrate der seriellen Schnittstelle
	#define USART_USE1		0		//!< 0 fr USART0 und 1 fr USART1 bei ATmega644P 
									// ACHTUNG bei 1 geht LCD am EXT Port nicht mehr Warum? keine Ahnung...
									//	 Kamera ist dann an der anderen Schnittstelle
	
//-----------------------------------------------------------------------
	// LCD einbinden 
	//-----------------------------------------------------------------------
	
	#define I2C_use				0	// I2C LCD mit PCF8574P benutzen/Porterweiterung ebenfalls PCF8574
	// Bei i2c Lcd Ende
	
	// LCD einbinden ber EXT. Anschlu
	#define USE_LCD				0	// LCD benutzen (alle Modi)
									// Der serielle Modus an PORT C wird benutzt wenn keine
									// der nchsten vier Einstellungen gesetzt wird
	#if USE_LCD
		#define LCD_4Bit		0	// LCD im 4Bit Mode an PORTC (1) oder seriell (0)
		#define LCD_EXT			0	// LCD im 4Bit Mode am Ext.Port des AVR-NET-IO
		#define LCD_EXT_POLLIN	0	// wie LCD_EXT mit Pollin Verdrahtung (Handbuch S.16)
		#define	LCD_EXT_SERIELL	0	// LCD im seriellen Modus am EXT-Port des AVR-NET-IO

		#define LCD_USER_CHARS	0	// sollen benutzerdefinierte Zeichen benutzt werden?
		#define LCD_ONE_LINE	0	// 0 = mehrzeiliges Display / 1 = 1-zeiliges Display
		#define	LCD_4x20		1	// 0 = alles kleiner als 4x20 / 1 = 4x20
		
		#define USE_LCD_UDP		0	// soll das Display ber UDP angesteuert werden?

		#define	LCD_BACKLIGHT	1	// LCD hat Hintergrundbeleuchtung (4Bit,EXT,EXT_POLLIN)

		#if LCD_BACKLIGHT
			#define	LCD_BL_AUTO	0	// 1 = Hintergrundbeleuchtung automatisch abschalten
			#define LCD_BL_TIME	15	// Sekunden nach denen Beleuchtung abgeschaltet wird
		#endif
	#endif


/** Porterweiterung**************/
	#define USE_PE		0	//Porterweiterungen ber PCF8574/setzt "#define I2C_use	1" voraus
	/*  #if USE_PE
		#define	PEE_DDR				DDRE
		#define PEE_OUTPORT			PORTE
		#define PEE_INPORT			PINE

		#define PEF_DDR				DDRF
		#define PEF_OUTPORT			PORTF
		#define PEF_INPORT			PINF
	  #endif	
		*/
				
/** ADC ***************/
	#define USE_ADC			1		//!< AD-Wandler benutzen?
	#define MAXADCHANNEL	3		//!< max. Anzahl der verwendeten Ports fr AD-Wandler (PA0 bis PAn; n = MAXADCHANNEL - 1)

/** 1-Wire ************/
	#define USE_OW			1		//!< 1-Wire bus einbinden
	#define MAXSENSORS		4		//!< max. Anzahl der 1-wire sensoren (DS18B20)
	#define OW_ONE_BUS		1		//!< nur ein fest eingestellter 1-wire bus; keine Auswahl
									//   die Ports werden in onewire.h definiert
	#define OW_EXTERN_POWERED 0		//!< falls der 1-Wire bus eine externe Versorgungsspannung hat
	#define OW_ROMCODE_SIZE 8		//!< rom-code DS18B20 size including CRC

	// hier die ausgelesenen ROM-IDs der DS18B20 eintragen
	#define OW_ID_T01	{0x10,0xe7,0x1d,0x9d,0x01,0x08,0x00,0x66}
	#define OW_ID_T02	{0x10,0xc6,0xf3,0x98,0x01,0x08,0x00,0x1e}
	#define OW_ID_T03	{0x10,0x5a,0xd7,0x9c,0x01,0x08,0x00,0x3b} // leer
	#define OW_ID_T04	{0x10,0x53,0x0c,0x99,0x01,0x08,0x00,0x02} // leer
	#define OW_ID_Last	{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}

/** Servo ************/

	#define USE_SERVO		0
/** Kamera **********/

	#define USE_CAM		0	

/** SD-Karte **********/
	// USE_MMC ist Voraussetzung fr
	// - TCP_SERVICE	FTP ohne Dateisystem ist nicht sinnvoll ;-)
	// - E_Mail			Mailtexte werden von Karte gelesen
	// - Scheduler		Konfiguration der Schaltzeiten auf SD-Karte
	// - Logdatei		8-))
	//
	#define USE_MMC			1		//!< mit/ohne SD-Karte
	#define MAX_PATH		63		//!< maximale Pfadlnge fr FAT16-Directory

/**  Display  *******************/
	#define USE_DISPLAY	0	//Hardwarenderungen am ADD-on notwendig!!



/** TCP-Service *******/
	#define TCP_SERVICE		1		//!< mit/ohne TCP-Service (FTP, Telnet-Cmdline)-----------------<
	#define MYTCP_PORT		61234	//!< Port# fr Telnet-Cmd Interpreter	
	#define DOS_LIST		1		//!< DOS style Directory-Listing
	//#define UNIX_LIST		0
	#define FTP_ANONYMOUS	0		//!< anonymen Login (ohne User/Kennwort) erlauben
	#define FTP_USER		"admin"	//!< FTP-User, falls nicht anonym
	#define FTP_PASSWORD	"neset"	//!< FTP-Passwort

	// don't touch!
	#ifdef DOS_LIST
		#undef UNIX_LIST
	#endif  

/** Passwort **********/
	#define HTTP_AUTH_DEFAULT	1	//!< Webserver mit Passwort? (0 == mit Passwort)
	
	//AUTH String "USERNAME:PASSWORT" max 14Zeichen
	//fr Username:Passwort
	#define HTTP_AUTH_STRING "admin:neset"
	
/** E-Mail ************/
    #define USE_MAIL			0			//!< sendmail verwenden
	#define MAIL_DATAFILE		"mail.ini"	//!< Datei fr E-Mail Runtime-Konfiguration

	// don't touch! -- Mail bentigt DNS-Auflsung und die SD-Karte
	#if USE_MAIL
	#undef	USE_DNS
	#define USE_DNS			1
	#endif

    
/** Telnet -> Befehlszeile oder USART *********/
    #define CMD_TELNET      	0			//!< Kommandos und Ausgaben erfolgen ber Telnet (Port 23)

/** Infrarot Fernbedienung **/
	#define USE_RC5				0
	#if USE_RC5
		#define	RC5_DDR				DDRB		//!< IR input port Data Direction Register
		#define	RC5_INPORT			PINB		//!< IR input port
		#define	RC5_PIN				PB0		//!< \port PB0 - IR input pin Add-On Board
	#endif
/** Scheduler *********/
    #define USE_SCHEDULER		1			//!< Scheduler verwenden
	#define	TM_MAX_SCHALTZEITEN	16			//!< Anzahl unterschiedlicher Schaltzeiten
	#define SCHED_INIFILE		"avr.ini"	//!< Dateiname fr Schaltzeiten
	#define TIME_TEMP			60			//!< jede Minute (60 sec) Temperaturen messen
	#define USE_LOGDATEI		1			//!< Logdatei schreiben

	#define MAX_VAR_ARRAY		12			//!< Anzahl Speicherpltze fr Messwerte etc.

/*------------------------------------*/
	
	// don't touch! -- siehe oben unter SD-Karte
	#if !USE_MMC
		#undef  TCP_SERVICE
		#define TCP_SERVICE 	0

		#undef  USE_MAIL
		#define USE_MAIL 		0

		#undef  USE_SCHEDULER
		#define USE_SCHEDULER	0

		#undef  USE_LOGDATEI
		#define USE_LOGDATEI	0

		#undef	USE_DNS
		#define USE_DNS			0
	#endif

	
	
	// don't touch! -- siehe oben unter LCD
		
	#if LCD_EXT					// LCD am Ext.Port
			#undef	LCD_4Bit
			#define LCD_4Bit		0	// LCD_EXT bentigt LCD_4Bit
		#endif
	#if LCD_EXT_POLLIN			// LCD am Ext.Port mit Pollin Verdrahtung
			#undef	LCD_4Bit
			#define LCD_4Bit		0	// LCD_EXT_POLLIN bentigt LCD_4Bit
			#undef	LCD_EXT
			#define LCD_EXT			0	// LCD_EXT_POLLIN bentigt LCD_EXT
		#endif
/*------------------------------------
*	typedefs
*/
#ifndef __STDINT_H_
	#include <stdint.h>
#endif

typedef enum { false, true } bool;

/**
 *	\ingroup main
 *	aktueller Zustand der Schalter und Zhler
 */
typedef struct
{
	volatile bool		relais1;			// Relais an/aus
	volatile bool		relais2;
	volatile bool		relais3;
	volatile uint8_t 	TReglerWert;
	volatile uint16_t 	Zaehler1;			// Zhlerwerte
	volatile uint16_t 	Zaehler2;
	volatile uint16_t 	Zaehler3;
} ANLAGEN_STATUS;

/**
 *	\ingroup main
 *	Zustandsbeschreibung der angeschlossenen Schalter und Relais
 */
typedef struct
{
	volatile uint8_t	Schalter1	: 1;	// Sollzustand an/aus
	volatile uint8_t	Schalter2	: 1;
	volatile uint8_t	Schalter3	: 1;
	volatile uint8_t 	TReglerWert;
} SOLL_STATUS;

/**
 *	\ingroup schedul
 *	Schaltaktion mit Wochentag und Uhrzeit
 */
typedef struct 
{
	volatile uint8_t		Uhrzeit;		//!< Tageszeit
	volatile uint8_t 		Wochentag;		//!< pro Tag ein bit
	volatile SOLL_STATUS	Zustand;		//!< Sollzustand der Schalter
} TM_Aktion;

/**	
 * \ingroup main
 * Prototyp fr Timerfunktion
 */
typedef void(*timercmd_fp)(void);


/**
 *	\ingroup main
 *	Prozessstatus und aktuelle Werte fr die Endlosschleife in main()
 */
typedef struct
{
	volatile uint8_t		timeChanged;	//!< Zeitmessung gendert
	volatile bool 			initialisieren;	//!< Maschine muss initialisiert werden
	volatile bool 			Tlesen;			//!< Temperaturen mssen gelesen werden
	volatile bool 			regeln;			//!< Werte einstellen
	volatile bool 			LogInit;		//!< SD-Karte mit Logdatei neu initialisieren
	volatile bool 			LogSchreiben;	//!< Werte mssen geloggt werden
	volatile bool			ADCgemessen;	//!< analoger Wert am ADC wurde gemessen
	volatile uint8_t		sendmail;		//!< # der zu sendenden E-Mail
	volatile unsigned char	PINAchanged;	//!< Eingang wurde gendert
	volatile unsigned char	PINAStatus;		//!< letzter Wert fr PINA
	volatile unsigned char	PINCStatus;		//!< letzter Wert fr PINC
	volatile unsigned char	PINDStatus;		//!< letzter Wert fr PIND
	volatile uint8_t 		Timer1;			//!< Countdown Timer #1
	volatile uint8_t 		Timer2;			//!< Countdown Timer #2
	timercmd_fp				Timer2_func;
	volatile uint8_t 		Timer3;			//!< Countdown Timer #3
	volatile bool			Time3Elapsed;	
	timercmd_fp				Timer3_func;
	#if USE_RC5
	volatile uint16_t 		lastRC5data;	//!< letzter gltiger RC5 Code
	int8_t					disp_index;		//!< aktueller angezeigter Wert im LCD
	#endif
} PROZESS_STATUS;

#if USE_MMC

#include "sdkarte/fat16.h"	// Definitionen fr Dateisystem-Strukturen

/**
 *	\ingroup sdkarte
 *	FAT Dateistruktur
 */
typedef struct fat16_file_struct
{
    struct fat16_fs_struct* fs;					//!<
    struct fat16_dir_entry_struct dir_entry;	//!<
    uint32_t pos;								//!< aktuelle Schreib-/Lese-Position in Datei
    uint16_t pos_cluster;						//!< aktueller Datei cluster im Puffer
	char	mode;								//!< file open mode
} File;

/**
 *	\ingroup main
 *	Status der Logdatei
 */
typedef struct
{
	const char			log_datei[13];	//!< aktueller Dateiname der Logdatei
	File 				*logfile;		//!< Zeiger auf FILE-Struktur der offenen Logdatei
	volatile uint8_t	logTag;			//!< Tag der Logdatei
	volatile uint8_t	logMonat;		//!< Monat der Logdatei
	volatile uint8_t	logJahr;		//!< Jahr der Logdatei
} LOG_STATUS;

#endif


/*------------------------------------
*	globale Variable
*/
	extern volatile ANLAGEN_STATUS	anlagenStatus;
	extern volatile PROZESS_STATUS 	machineStatus;
#if USE_MMC
	extern 			LOG_STATUS		logStatus;
#endif
	extern volatile SOLL_STATUS 	anlagenSollStatus;
	extern unsigned int var_array[MAX_VAR_ARRAY];

#endif //_CONFIG_H


/*
 */
