
/**
 * \addtogroup disp
 *
 * @{
  */

/**
 * \file
 * disp.c
 */
/*This is a link library for the Siemens S65, M65, SX65 and RX65 mobile phone display.

This port is part of  copyright 2005 by Christian Kranz.

Based on the glcd copyright 2004 Hagen Reddmann, HaReddmann at T-Online dot de
*/
/**
 * @}
 */

#include "config.h"
//#include "../sdkarte/spi.h"

#include <stdint.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

//#include "networkcard/enc28j60.h"
#include "disp.h"
#include "sdkarte/fat16.h"
#include "sdkarte/sdcard.h"
//#include "nasa256c.h"

int i,j;
#define DISP_DDR DDRD
#define GLUE(a, b)     a##b
#define PORT(x)        GLUE(PORT, x)
#define PIN(x)         GLUE(PIN, x)
#define DDR(x)         GLUE(DDR, x)

char picname[12];

#define DISP_RESET 2		// Reset Display 
#define DISP_SELECT 6	// Cable Select 
#define DISP_DC 4		// Data/Command 
//#define DISP_CLK()   DDRB |= (1 << DDB7)

#define SET(bit) (PORTD |= _BV(bit))
#define RESET(bit) (PORTD &= ~_BV(bit))

#define SPI_DDR                 DDRB
	#define SPI_PORT                PORTB
	#define SPI_SCK                 7
	#define SPI_MISO                6
	#define SPI_MOSI                5
	


#define bright_blue		0b1101111011011111	
#define blue			0b0000000000011111
#define dark_blue		0b0000000000010011
#define bright_yellow		0b1111111111001100	
#define yellow			0b1111111111100000	
#define orange			0b1111110011000110
#define bright_red		0b1111100011100011
#define red			0b1111100000000000
#define dark_red		0b1001100000000000
#define bright_green 		0b1001111111110011
#define green 			0b0000011111100000
#define dark_green 		0b0000001101100000
#define white 			0b1111111111111111
#define grey 			0b0011100011100111
#define black 			0b0000000000000000


void DISP_test(void)
{	
	SPI_PORT &= ~(1<<SPI_SCK);	delay_ms(75);		
		for(unsigned char i=0; i<=131; i++){
		DISP_Plotd(i,88,blue);
		}
	DISP_Box(5,5, 25, 25,green);
	/*DISP_Box(5,35,25,55,orange);
	DISP_Box(5,65,25,85,red);
	DISP_Box(100,5,125,25,green);
	DISP_Box(100,35,125,55,orange);*/
	DISP_Box(100,65,120,85,red);			//(x1,y1,x2,y2,color) 
	delay_ms(2000);

/*
#if USE_MMC
	DISP_Window(0,0, 131,175);
	strcpy(picname, "17.bmp");
	S65_PutBmpFromSD(0,0, &picname[0]);
#endif
*/
	DISP_Window(35,35, 53,51);
	Move(35,35);
	//DISP_Plot(38,90,dark_red);
	//Bmp(BitmapSmall, 288,grey,bright_red);
	//LCD_Bitmap_256high(0, 0, 131, 175, Nasa256c, Table_Nasa256c, true);
	DISP_Box(5,65,25,85,red);	
	SPI_PORT |= (1<<SPI_SCK);	delay_ms(75);
}

//-----------------------------------------------------------------------------------
void delay_ms(uint16_t period)	 //delay routine (milliseconds)
{
	for(uint16_t i=0; i<=period; i++)
		_delay_ms(1);
}

void DISP_Init(void)
{
	//display_spi_init();
	SPI_DDR |= (1<<SPI_MOSI) | (1<<SPI_SCK);
	SPI_DDR &= ~(1<<SPI_MISO);

	//SPI: enable, master, positive clock phase, msb first, SPI speed fosc/2
	SPCR = (1<<SPE) | (1<<MSTR);
	SPSR = (1<<SPI2X);


	DDRD = 0x76;
	delay_ms(100); 
	RESET(DISP_RESET);	delay_ms(75);
	SET(DISP_SELECT);	delay_ms(75);
	SPI_PORT &= ~(1<<SPI_SCK);		delay_ms(75); 
	SET(DISP_DC);		delay_ms(75); 
	SET(DISP_RESET);		delay_ms(75); 


	unsigned int init_data[] = 
	{ 
		0xFDFD, 0xFDFD, 
		// pause 
		0xEF00, 0xEE04, 0x1B04, 0xFEFE, 0xFEFE, 
		0xEF90, 0x4A04, 0x7F3F, 0xEE04, 0x4306, 
		// pause 
		0xEF90, 0x0983, 0x0800, 0x0BAF, 0x0A00, 
		0x0500, 0x0600, 0x0700, 0xEF00, 0xEE0C, 
		0xEF90, 0x0080, 0xEFB0, 0x4902, 0xEF00, 
		0x7F01, 0xE181, 0xE202, 0xE276, 0xE183, 
		0x8001, 0xEF90, 0x0000 
	}; 
	DISP_SPI_DBCommand(&init_data[0], 2);	delay_ms(75); 
	DISP_SPI_DBCommand(&init_data[2], 10);	delay_ms(75); 
	DISP_SPI_DBCommand(&init_data[12], 23); delay_ms(75);

	DISP_Cls(yellow);	delay_ms(75); 
	RESET(DISP_SELECT);	delay_ms(75);
	SPI_PORT |= (1<<SPI_SCK);
	delay_ms(75); 
}

void DISP_Window(int x1, int y1, int x2, int y2) 
{ 
	unsigned char data[] = 
	{ 
		0xEF, 0x08,
		0x18, 0x00,
		0x12, x1,
		0x15, x2,
		0x13, y1,
		0x16, y2 
	};
	SET(DISP_DC);					// DataCmd := 1 
	DISP_SPI_Byte(data, 12); 
	SET(DISP_SELECT);
	RESET(DISP_DC);					// DataCmd := 0 
	RESET(DISP_SELECT);
} 
/*void Bmp( const unsigned char Pbmp[], unsigned int PixelCount,unsigned int bcolor,unsigned int vcolor)
{
	 unsigned int b;
	for(unsigned int i=0; i<=PixelCount; i++)
	{
		b = pgm_read_byte(&Pbmp[i]);		//const unsigned int color
							//DISP_SPI_Int(color);
	 						//int i,j;
   
                    if((b == 1)) DISP_SPI_Int(vcolor);
                    else DISP_SPI_Int(bcolor);
			SET(DISP_SELECT);		// CSelect := 1 
			RESET(DISP_SELECT);		// CSelect := 0 
              }
}
*/
void Move(int x,int y) {

 // int i,j;

  for(i=0; i<16; i++) {
    for(j=0; j<18; j++) {
        DISPPixel((x+j), (y+i), BitmapSmall[i][j],grey,red);		
    }
  }
}
void DISPPixel (unsigned char x, unsigned char y, const unsigned char Pbmp[],unsigned int bcolor,unsigned int fcolor )
{
	//int i,j;
	unsigned char modea;
	modea = pgm_read_byte(&Pbmp);
    if ( modea == PIXEL_OFF )
    {
        DISP_SPI_Int( bcolor);
    }
    else 
    {
        DISP_SPI_Int( fcolor);
    }
    
	SET(DISP_SELECT);		// CSelect := 1 
	RESET(DISP_SELECT);		// CSelect := 0 
}
/*

void LCD_Bitmap_256high(int x1, int y1, int x2, int y2, const prog_uint8_t Pbmp[], const prog_uint16_t ColorTable[], bool Compressed)
{
	LCD_Window(x1, y1, x2, y2);    // Define Window	
	
	const int Width = x2 - x1 + 1;
	const int Height = y2 - y1 + 1;
	const unsigned int PixelCount = Width * Height;

	if (Compressed)
		Bitmap_256high_compressed(Pbmp, ColorTable, PixelCount);
	else
		Bitmap_256high_uncompressed(Pbmp, ColorTable, PixelCount);

	SET(LCD_SELECT);				 // CSelect := 1 
	RESET(LCD_SELECT);	
}
*/
//  DISP_Cls: Clear Screen: 
void DISP_Cls(unsigned int color) 
{ 
	DISP_Window(0, 0, 131, 175);
	for (unsigned int i=0; i<23232; ++i)
	{ 
		DISP_SPI_Int(color); 
		SET(DISP_SELECT);			//CSelect := 1 
		RESET(DISP_SELECT);			// CSelect := 0 
	}
} 

void DISP_Plot(unsigned char x1, unsigned char y1, unsigned int color)
{
	DISP_Window(x1, y1, x1, y1); 		// Define Window, exactly one pixel large
	DISP_SPI_Int(color);			// send pixel
	SET(DISP_SELECT);			// CSelect := 1 
	RESET(DISP_SELECT);			// CSelect := 0 
}

void DISP_Plotd(unsigned char x1, unsigned char y1, unsigned int color)        //d-für "dick"
{
		DISP_Window(x1, y1, x1+1, y1+1); 	// Define Window, exactly four pixels large
		DISP_SPI_Int(color);			// send four pixels (avoid loop overhead)
		DISP_SPI_Int(color);
		DISP_SPI_Int(color);
		DISP_SPI_Int(color);
		SET(DISP_SELECT);			// CSelect := 1 
		RESET(DISP_SELECT);			// CSelect := 0 
}

/*void DISP_Rect(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char line_type, unsigned int color)	//draw rectangle
{
      DISP_Draw(x1, y1, x2, y1, line_type, color);
      DISP_Draw(x2, y1, x2, y2, line_type, color);
      DISP_Draw(x1, y2, x2, y2, line_type, color);
      DISP_Draw(x1, y1, x1, y2, line_type, color);
}
*/
void DISP_Box(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned int color)
{
	DISP_Window(x1, y1, x2, y2);   										// Define Window
	for (unsigned int i=0; i<((x2-x1+1)*(y2-y1+1)); ++i)			// for every pixel...
		DISP_SPI_Int(color); 											// ...send color information
	SET(DISP_SELECT);													// CSelect := 1 
	RESET(DISP_SELECT);													// CSelect := 0 
}

// send single bytes (or array with Bytes) by SPI 
void DISP_SPI_Byte(const unsigned char data[], unsigned int count) 
{ 
	//....data 
	for (unsigned int i=0; i<count; ++i)	// Count Bytes 
	{
		SPCR |= _BV(SPE);
		SPDR = data[i];
		DISP_Wait();
	} 
}
// send word (16 bit) (or array with words) by SPI 
void DISP_SPI_DBCommand(const unsigned int data[], unsigned int count)
{
	for (int i=0; i<count; ++i)
	{
		RESET(DISP_SELECT);					// CSelect := 0 
		SET(DISP_DC);						// DataCmd := 1 
 		unsigned char msbyte = (data[i] >> 8) & 0xff;		// msb!
		unsigned char lsbyte = data[i] & 0xff;			// lsb!	
		SPCR |= _BV(SPE);
		SPDR = msbyte;						//send first 8 bits
		DISP_Wait();
		SPCR |= _BV(SPE);
		SPDR = lsbyte;						//send last 8 bits
		DISP_Wait();
		RESET(DISP_DC);						// DataCmd := 0 
	}
}

void DISP_SPI_Int(unsigned int Value) 
{ 
	SPCR |= _BV(SPE);
	SPDR = (Value >> 8) & 0xff;
	DISP_Wait();
	SPCR |= _BV(SPE);
	SPDR = Value & 0xff;
	DISP_Wait();
}

void DISP_Wait(void){						//waits for SPI transfer is done

    while (SPCR & _BV(SPE)) {
        while (!(SPSR & (_BV(SPIF))));
        SPCR &= ~(_BV(SPE));
    }
}

const unsigned char BitmapSmall[][18]  = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,
  0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0,
  0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0,
  0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 0, 0,
  0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,
  0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0,
  0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0,
  0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

/*
static void testBMP(uint8_t __attribute__ ((progmem)) eye[], unsigned int PixelCount)
{
	for(unsigned int i=0; i<PixelCount; ++i)
	{
		const unsigned int PixelColor = pgm_read_word(&Pbmp[i]);
		DISP_SPI_Byte(PixelColor);
	}
}
*/
/*
// paint own bitmaps(see copyrights above)
void testBitmaps(void) {

    uint8_t Bitmap[512];
    glcdCoord_t width = glcd_Clip.X2 - glcd_Clip.X1 +1;
    glcdCoord_t height = glcd_Clip.Y2 - glcd_Clip.Y1 +1;
    int16_t i,j;

    for (i = 0; i < 512; i++) {      // create a bitmap
      Bitmap[i] = i * 2;
    }

    glcdSetAddr(glcd_Clip.X1, glcd_Clip.Y1, glcd_Clip.X2, glcd_Clip.Y2);  // bounds of bitmap on screen

    for (i = 0; i < 256; i++) {
      for (j = 0; j < height; j++) {
        glcdDisplayBuffer(&Bitmap[i + j], width, 0);
      }
    }

}
*/

////////////////////////////////////////////////
/*#if USE_MMC
int	S65_PutBmpFromSD(uint8_t x, uint8_t y, char *Bildname)		// x, y Startkoordinaten des Bildes
{
	//Load_BMP

	uint16_t bftype = 0;
	int16_t bmpWidth = 0;
	int16_t bmpHeight = 0;
	int16_t Offsetbmp;
	int8_t farbtiefe = 0;
//	int8_t compression = 0;		// Wird im moment nicht verwendet
	bmpWidth = 0x00;		// Höhe des Bildes
	bmpHeight = 0x00;		// Breite des Bildes
	Offsetbmp = 0x00;
	uint8_t bisizeimagea = 0;
	uint16_t pixel = 0;
	uint16_t pixeltemp = 0;
	uint16_t cetemp = 0;
	uint16_t anfang = 0;
	
	mmc_read_csd (Buffer);

	//Lade Cluster für das File in den Speicher 
	Clustervar = 0;//suche im Root Verzeichnis
	if (fat_search_file(Bildname,&Clustervar,&Size,&Dir_Attrib,Buffer) == 1)
		{
		fat_read_file (Clustervar,Buffer,0);
			_delay_ms(400);
			
			//bftype prüfen
			bftype = Buffer[1];
			bftype = bftype << 8;
			bftype = bftype + Buffer[0];
			if (bftype == 19778)
			{
				// usart_write("bfType ist OK. \n");
			}
			else
			{
				// usart_write("bfType ist NICHT OK. \n");
				return(0);
			}

			//Prüfe Offset
			Offsetbmp = Buffer[11];
			Offsetbmp = Offsetbmp << 8;
			Offsetbmp = Offsetbmp + Buffer[10];

			bmpWidth = Buffer[18];
			bmpHeight = Buffer[22];
			
			bisizeimagea = Buffer[34];
			farbtiefe = Buffer[28];
			// compression = Buffer[30];		// Wird im moment nicht verwendet
			
			S65_allocatearea2(0 ,0 ,0 ,176);


		//Konfiguert den Speicherzugriff, und legt den Bild Bereich fest.
		S65_wrcmd16(0xEF90);		S65_wrcmd16(0x0504);
		LCD_WrCmd(0x08);		LCD_WrCmd(x);
		LCD_WrCmd(0x09);		LCD_WrCmd(x + bmpHeight - 1);
		LCD_WrCmd(0x0A);		LCD_WrCmd(y);
		LCD_WrCmd(0x0B);		LCD_WrCmd(y + bmpWidth - 1);
		
		pixel = (bmpHeight * bmpWidth);		// Anzahl der Pixel

// 16 Bit Farbtiefe
			if (farbtiefe == 16)
			{
				// Ermittelt, wieviel Datenpakete entstehen
				pixeltemp = (pixel - ((512 - Offsetbmp) / 2));	
				
				// ermittelt wieviele Pixel im letzen Paket wichtig sind
				for (int ce = 1; pixeltemp>256; ce++)
				{
					pixeltemp = pixeltemp - 256;
					cetemp = ce + 1;
				}
				
				pixeltemp = pixeltemp * 2;

				CLR(LCD_CS);					//select Display
				CLR(LCD_RS);					//clear RS line
				for (int a = Offsetbmp;a<512;a+=2)
					{
						SPDR = Buffer[a+1];	// erste acht daten bits
						while(!(SPSR & (1<<SPIF)));
						SPDR = Buffer[a];		// zweiten acht bits
						while(!(SPSR & (1<<SPIF)));
					}
				SET(LCD_CS);

			// Liest die restlichen Bytes
				for (int b = 1;b<(cetemp);b++)
					{
					fat_read_file (Clustervar,Buffer,b);
					CLR(LCD_CS);					//select Display
					CLR(LCD_RS);					//clear RS line
					for (int a = 0;a<512;a+=2)
						{
							SPDR = Buffer[a+1];	// erste acht daten bits
							while(!(SPSR & (1<<SPIF)));
							SPDR = Buffer[a];		// zweiten acht bits
							while(!(SPSR & (1<<SPIF)));
						}
					SET(LCD_CS);
					}

			// Das letze Cluster wird ausgegeben.
					fat_read_file (Clustervar,Buffer,(cetemp));
					CLR(LCD_CS);					//select Display
					CLR(LCD_RS);					//clear RS line

					for (int a = 0;a<(pixeltemp);a+=2)
						{
							SPDR = Buffer[a+1];	// erste acht daten bits
							while(!(SPSR & (1<<SPIF)));
							SPDR = Buffer[a];		// zweiten acht bits
							while(!(SPSR & (1<<SPIF)));
						}
				SET(LCD_CS);
			}


// --------------------------------------------------------
			// Es sind 8 Bit Farbtiefe
			else if (farbtiefe == 8)   //8 bit Farbtiefe
			{
				while (Offsetbmp > 512)
				{
					Offsetbmp = Offsetbmp - 512;
					anfang ++;
				}

				fat_read_file (Clustervar,Buffer,anfang);

				pixeltemp = (pixel - ((512 - Offsetbmp)));	
				for (int ce = 1; pixeltemp>512; ce++)
				{
					pixeltemp = pixeltemp - 512;
					cetemp = ce;
				}
				cetemp = cetemp + anfang;
				S65_8bit_data();	// Konfigueriert das Display im 8 Bit Farb modus

				//______________________________________________
				//Konfiguert den Speicherzugriff, und legt den Bild Bereich fest.
				S65_wrcmd16(0xEF90);		S65_wrcmd16(0x0504);
				LCD_WrCmd(0x08);		LCD_WrCmd(x);
				LCD_WrCmd(0x09);		LCD_WrCmd(x + bmpHeight - 1);
				LCD_WrCmd(0x0A);		LCD_WrCmd(y);
				LCD_WrCmd(0x0B);		LCD_WrCmd(y + bmpWidth - 1);
				//___________________________________________________
				
				CLR(LCD_CS);					//select Display
				CLR(LCD_RS);					//clear RS line
				for (int a = Offsetbmp;a<512;a++)
					{
						SPDR = Buffer[a];		// acht bits
						while(!(SPSR & (1<<SPIF)));
					}
				SET(LCD_CS);
				for (int b = anfang + 1;b<(cetemp+1);b++)
					{
					fat_read_file (Clustervar,Buffer,b);
					CLR(LCD_CS);					//select Display
					CLR(LCD_RS);					//clear RS line

					for (int a = 0;a<512;a++)
						{
							SPDR = Buffer[a];		// acht bits
							while(!(SPSR & (1<<SPIF)));
						}
					SET(LCD_CS);
					}

			// Das letze Cluster wird ausgegeben.
					fat_read_file (Clustervar,Buffer,(cetemp+1));
					CLR(LCD_CS);					//select Display
					CLR(LCD_RS);					//clear RS line

					for (int a = 0;a<(pixeltemp);a++)
						{
							SPDR = Buffer[a];		// acht bits
							while(!(SPSR & (1<<SPIF)));
						}
				SET(LCD_CS);
				S65_16bit_data();

				return(0);
			}
			else	// Wenn Farbtiefe nicht 16 Bit und nicht 8 Bit, liegt ein fehler vor.
			{
				return(1);
			}
		}
return(1);		// Alles klar, keine Fehler
//Load_BMP_Ende	
}
*/
/*
void S65_16bit_data()
{
	S65_wrcmd16(0xEF90);
	S65_wrcmd16(0xE80F);
	S65_refresh();
}

void S65_8bit_data()
{
	S65_wrcmd16(0xEF90);
	S65_wrcmd16(0xE800);
	//S65_wrcmd16(0xE801);
	S65_refresh();
}

void S65_refresh()
{
	S65_wrcmd16(0xEF90);
	S65_wrcmd16(0x0000);
}
*/

/**
 * @}
 * @}
 */

//#endif
