/************************************************************************
**
**  Copyright (C) 2005  Christian Kranz
**
**  Siemens S65 Display Control
**
**  modded by Juergen Eckert
**
**  re-moddes by Marc Seiffert 
**
**  Circle-module given by Harry Sk. developed by unknown
**
************************************************************************/

#include "s65.h"
#include <avr/io.h> 
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "fat.h"
#include "mmc.h"
#include "typedefs.h"

#include <stdio.h>
#include <string.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <avr/eeprom.h>

#define ASX 0x1200
#define ASY 0x1300
#define APX 0x1400
#define AEX 0x1500
#define AEY 0x1600
#define APY 0x1700

// GLOBALE VARIABLEN, benutzervernderbar
uint8_t 	*FontPointer; 	//FontPointer auf Font
uint8_t 	rot;			//Ausrichtung der Schrift 0=0, 1=90
uint16_t 	TextFarbe;		//Textfarbe, 5-6-5 RGB
uint16_t 	HgFarbe;		//Textfarbe, 5-6-5 RGB
uint8_t 	TextTyp;		//Texttyp 0=Proportional , 1=Fixed	
uint8_t		FNull;			// Fhrende Nullen bei Zahlen anzeigen	0=Aus , 1=an
uint8_t		NonTransparenz; // Transparenz der Schriftarten. Wenn aktiv ist Weiss =  Transparent   0=An , 1=Aus
uint8_t		BmpTransparenz;	// Transparenz bei Bitmaps --> Achtung, verschlechtert die Zeichenperformance um den Faktor 7 !
uint8_t		BmpTransFarbe;	// Farbe, die transparent dargestellt wird
// GLOBAELE VARIABLEN, zur internen Benutzung
uint8_t		TextBreite;		// Maximale Textbreite der aktuellen Font
char		za[20];			// Zahlenzwischenspeicher
// GLOBALE VARIABLEN ende
unsigned char Buffer[512];	// Speicher fr grere Bildinhalte z.B. BMP
unsigned int Clustervar;
unsigned char Dir_Attrib = 0;
unsigned long Size = 0;

// Initialisierungssequenzen
prog_uint8_t INIT1[24] = {
0xFD, 0xFD, 0xFD, 0xFD, 0xEF, 0x00, 0xEE, 0x04, 
0x1B, 0x04, 0xFE, 0xFE, 0xFE, 0xFE, 0xEF, 0x90, 
0x4A, 0x04, 0x7F, 0x3F, 0xEE, 0x04, 0x43, 0x06,};
prog_uint8_t INIT2[40] = {
0xEF, 0x90, 0x09, 0x83, 0x08, 0x00, 0x0B, 0xAF, 
0x0A, 0x00, 0x05, 0x00, 0x06, 0x00, 0x07, 0x00, 
0xEF, 0x00, 0xEE, 0x0C, 0xEF, 0x90, 0x00, 0x40, 
0xEF, 0xB0, 0x49, 0x02, 0xEF, 0x00, 0x7F, 0x01, 
0xE1, 0x81, 0xE2, 0x02, 0xE2, 0x76, 0xE1, 0x83,};
prog_uint8_t INIT3[6] = {0x80, 0x01,0xEF,0x90,0x00,0x00};
// Initialisierungssequenzen ende

// HG-Bel-Wertetabelle
//prog_uint8_t HG[11] = {0,60,100,130,150,170,180,185,190,195,200};

void DISP_WrCmd(unsigned char data){
	CLR(DISP_SCK);	mswait(75);
	CLR(DISP_CS);					//select Display
	SET(DISP_RS);					//set RS line
	SPDR = data;
	while(!(SPSR & (1<<SPIF)));
	SET(DISP_CS);
	SET(DISP_SCK);	mswait(75);//hier gendert!!!!!!!!!!
}

void DISP_WrData(unsigned char data){
	CLR(DISP_SCK);	mswait(75);
	CLR(DISP_CS);					//select Display
	CLR(DISP_RS);					//clear RS line
	SPDR = data;
	while(!(SPSR & (1<<SPIF)));
	SET(DISP_CS);
	SET(DISP_SCK);	mswait(75);//hier gendert!!!!!!!!!!!
}

void mswait(unsigned int ms){
	unsigned int i;
	for(i=0; i<ms; i++)
		_delay_ms(1);
}

void wrtab(unsigned char* data, unsigned char size){
	unsigned char i;
	for(i=0; i<size; i++)
	{
		DISP_WrCmd(pgm_read_word_near(&data[i]));
	}
}

void S65_Init(void){
	//An dieser Stelle eine kurze Pause...

	OUTPUT(DISP_RESET);				//set reset as output
	CLR(DISP_RESET);					//Display reset
	INPUT(DISP_MISO);				//MISO is always input in SPI
	SET(DISP_MISO);					//enable pull up of MISO to avoid floating input
	OUTPUT(DISP_MOSI);				//MOSI must be set as output for SPI
	OUTPUT(DISP_CS);					//CS must be output for Master mode to work
	SET(DISP_CS);					//deselect display 
	OUTPUT(DISP_SCK);				//set SCK as output
	CLR(DISP_SCK);					//set SCK lo
	OUTPUT(DISP_RS);					//RS signal is output
	SET(DISP_RS);					//start with RS=high
		SPCR = (1<<MSTR) | (1<<SPE);//Master Mode,  Enable SPI, fCPU/4 clock
		SPSR |= (1<<SPI2X);			//double speed bit
	mswait(1);
		SET(DISP_RESET);				//clear status
		wrtab(INIT1, 24);
	mswait(75);
		wrtab(INIT2, 40);
	mswait(50);
		wrtab(INIT3, 6);
}

void S65_wrcmd16(uint16_t dat){DISP_WrCmd((dat>>8));DISP_WrCmd(dat);}

void S65_wrdat16(uint16_t dat){DISP_WrData(dat>>8);DISP_WrData(dat);}
//DISP_Window
void S65_allocatearea(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2){
	uint8_t x,y;
	if(x1 < x2){ x = x2; x2 = x1; x1 = x;} // x2 muss grer sein als x1
	if(y2 < y1){ y = y1; y1 = y2; y2 = y;} // x2 muss grer sein als x1
	S65_wrcmd16(0xEF80);		S65_wrcmd16(0x1805);
	S65_wrcmd16(0x1200 + x1);	S65_wrcmd16(0x1500 + x2);
	S65_wrcmd16(0x1300 + y1);	S65_wrcmd16(0x1600 + y2);
}

void S65_allocatearea2(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2){
//	uint8_t x,y;
	//if(x1 < x2){ x = x2; x2 = x1; x1 = x;} // x2 muss grer sein als x1
	//if(y2 < y1){ y = y1; y1 = y2; y2 = y;} // x2 muss grer sein als x1
	S65_wrcmd16(0xEF80);		S65_wrcmd16(0x1804);
	S65_wrcmd16(0x1200 + x1);
	S65_wrcmd16(0x1500 + x1 + (131));
	S65_wrcmd16(0x1300 + y1);
	S65_wrcmd16(0x1600 + y1 + (175));
}
//DISP_Cls
void S65_Fillscreen(uint16_t color)
{
	// Function optimiert fr Geschwindigkeit !!!
	// Es werden vor dem Senden der Daten die Bits gesetz, damit das display angesteuert werden kann.
	// Erst nachdem die Daten gesendet wurden, wird das ChipSelect signal wieder resetet.
  	S65_wrcmd16(0xEF90);  	S65_wrcmd16(0x0500);
  	S65_wrcmd16(0x0600);  	S65_wrcmd16(0x0700);
	CLR(DISP_CS);					//select Display
	CLR(DISP_RS);					//clear RS line

	for (uint16_t i=0; i<S65_DISP_W*S65_DISP_H; i++)
	{
		SPDR = color>>8;	// erste acht daten bits
		while(!(SPSR & (1<<SPIF)));
		SPDR = color;		// zweiten acht bits
		while(!(SPSR & (1<<SPIF)));
	}
	SET(DISP_CS);
}
//DISP_Plot
void S65_Pixel(uint8_t y, uint8_t x, uint16_t color){
	S65_allocatearea(x,y,x,y);
	S65_wrdat16(color);
}
/******************************************************************/
/*void 	S65_SetFont(uint8_t *pointer){FontPointer = pointer;TextBreite = pgm_read_byte_near(&FontPointer[2]);}
void 	S65_SetRot(uint8_t r){rot = r;};
void 	S65_SetTextColor(uint16_t color){TextFarbe = color;};
void 	S65_SetTextTyp(uint8_t typ){TextTyp = typ;}
void 	S65_SetFNull(uint8_t fn){FNull = fn;};			*/
void	S65_SetNonTransparenz(uint8_t nt){NonTransparenz = nt;};
void	S65_SetBmpTransparenz(uint8_t bt){BmpTransparenz = bt;};
void 	S65_SetBmpTransFarbe(uint16_t color){BmpTransFarbe = color;};
/*
uint8_t S65_PutChar(uint8_t x, uint8_t y, uint8_t c) // Ein Zeichen ausgeben, vorher muss eine Schriftart geladen werden
{
	uint8_t 	hoehe 			=	pgm_read_byte_near(&FontPointer[3]);
	uint8_t 	firstchar 		=	pgm_read_byte_near(&FontPointer[5]);
	uint8_t 	lastchar 		=	pgm_read_byte_near(&FontPointer[6]);
	uint8_t 	zeichenbreite 	= 	pgm_read_byte_near(&FontPointer[7+c-firstchar]);
	uint8_t 	byte 			= 	0;
	uint8_t 	bitoffset 		= 	0;
	uint8_t 	maske 			= 	0;
	uint8_t 	bcounter 		= 	0;
	uint16_t 	bitsbischar 	= 	0;
	uint16_t 	bytesbischar	= 	0;
	uint16_t 	counter 		= 	0;

	for(uint8_t Ccounter = 0; Ccounter < c-firstchar; Ccounter++)
		{
		bitsbischar += (pgm_read_byte_near(&FontPointer[Ccounter+7]));
		};

	bitsbischar *= hoehe;
	bitsbischar += ((lastchar-firstchar)+7)*8;
	bytesbischar = bitsbischar/8;
	bitoffset = bitsbischar % 8;
	maske = bitoffset % 8;

	for(uint8_t xc = 0; xc < zeichenbreite; xc++) // Eigentlcihe Ausgaberoutine
		{
		for(uint8_t yc = hoehe; yc > 0; yc--)
			{
			if(maske > 7){maske = 0;bcounter+=1;};
			byte = pgm_read_byte_near(&FontPointer[bytesbischar + bcounter]+1);
			
			if(bit_is_set(byte,maske))
				{
				if(rot)
					{
					S65_Pixel((hoehe-yc)+y,xc+x, TextFarbe);
					}
				else
					{
					S65_Pixel(xc+x,yc+y, TextFarbe);
					}
				}
			else
				{
				if(NonTransparenz)
				{
					if(rot)
						{
						S65_Pixel((hoehe-yc)+y,xc+x, HgFarbe);
						}
					else
						{
						S65_Pixel(xc+x,yc+y, HgFarbe);
						}
					}
				}

			maske++;
			counter++;
			}
		}
	return(zeichenbreite);
}

uint8_t	S65_PutStr(uint8_t x, uint8_t y, char *c) // Einen Nulterimierten String ausgeben
{	
	char zeichen = 0;
	uint8_t zeichenbreite = 0;
	uint8_t xo = 0;
	uint8_t counter = 0;
	while(c[counter] != 0)
		{
		zeichen = c[counter];
		zeichenbreite = S65_PutChar(x+xo,y,zeichen);
		if(TextTyp == 0)
			{
			xo += zeichenbreite+2;
			}
		else
			{
			xo+=TextBreite;
			}
		counter++;
		if(counter == 255){return(255);} // 255 ist "Fehleroode" fr unterminierten String

		}
	return(xo);
	}
*/
void S65_PutRawBmp16(uint8_t x, uint8_t y, uint8_t *pic, uint8_t w, uint8_t h)
	{
	uint8_t dimx = w-1; 
	uint8_t dimy = h-1;
	S65_allocatearea(x,y,x+dimx, y+dimy);
	for(uint16_t counter = 0; counter < ((dimx+1)*(dimy+1))*2; counter++)
		{
		DISP_WrData(pgm_read_byte_near(&pic[counter]));
		}
	}


/////////////////////////////////////////////////

int	S65_PutBmpFromSD(uint8_t x, uint8_t y, char *Bildname)		// x, y Startkoordinaten des Bildes
{
	//Load_BMP

	uint16_t bftype = 0;
	int16_t bmpWidth = 0;
	int16_t bmpHeight = 0;
	int16_t Offsetbmp;
	int8_t farbtiefe = 0;
//	int8_t compression = 0;		// Wird im moment nicht verwendet
	bmpWidth = 0x00;		// Hhe des Bildes
	bmpHeight = 0x00;		// Breite des Bildes
	Offsetbmp = 0x00;
	uint8_t bisizeimagea = 0;
	uint16_t pixel = 0;
	uint16_t pixeltemp = 0;
	uint16_t cetemp = 0;
	uint16_t anfang = 0;
	
	mmc_read_csd (Buffer);  //------------------------------------------------------------------

	//Lade Cluster fr das File in den Speicher 
	Clustervar = 0;//suche im Root Verzeichnis
	if (fat_search_file(Bildname,&Clustervar,&Size,&Dir_Attrib,Buffer) == 1)
		{
		fat_read_file (Clustervar,Buffer,0);
			_delay_ms(400);
			
			//bftype prfen
			bftype = Buffer[1];
			bftype = bftype << 8;
			bftype = bftype + Buffer[0];
			if (bftype == 19778)
			{
				// usart_write("bfType ist OK. \n");
			}
			else
			{
				// usart_write("bfType ist NICHT OK. \n");
				return(0);
			}

			//Prfe Offset
			Offsetbmp = Buffer[11];
			Offsetbmp = Offsetbmp << 8;
			Offsetbmp = Offsetbmp + Buffer[10];

			bmpWidth = Buffer[18];
			bmpHeight = Buffer[22];
			
			bisizeimagea = Buffer[34];
			farbtiefe = Buffer[28];
			// compression = Buffer[30];		// Wird im moment nicht verwendet
			
			S65_allocatearea(0 ,0 ,131 ,175);//------------?2(0,0,0,176)


		//Konfiguert den Speicherzugriff, und legt den Bild Bereich fest.
		S65_wrcmd16(0xEF90);		S65_wrcmd16(0x0504);
		DISP_WrCmd(0x08);		DISP_WrCmd(x);
		DISP_WrCmd(0x09);		DISP_WrCmd(x + bmpHeight - 1);
		DISP_WrCmd(0x0A);		DISP_WrCmd(y);
		DISP_WrCmd(0x0B);		DISP_WrCmd(y + bmpWidth - 1);
		
		pixel = (bmpHeight * bmpWidth);		// Anzahl der Pixel

// 16 Bit Farbtiefe
			if (farbtiefe == 16)
			{
				// Ermittelt, wieviel Datenpakete entstehen
				pixeltemp = (pixel - ((512 - Offsetbmp) / 2));	
				
				// ermittelt wieviele Pixel im letzen Paket wichtig sind
				for (int ce = 1; pixeltemp>256; ce++)
				{
					pixeltemp = pixeltemp - 256;
					cetemp = ce + 1;
				}
				
				pixeltemp = pixeltemp * 2;

				CLR(DISP_CS);					//select Display
				CLR(DISP_RS);					//clear RS line
				for (int a = Offsetbmp;a<512;a+=2)
					{
						SPDR = Buffer[a+1];	// erste acht daten bits
						while(!(SPSR & (1<<SPIF)));
						SPDR = Buffer[a];		// zweiten acht bits
						while(!(SPSR & (1<<SPIF)));
					}
				SET(DISP_CS);

			// Liest die restlichen Bytes
				for (int b = 1;b<(cetemp);b++)
					{
					fat_read_file (Clustervar,Buffer,b);
					CLR(DISP_CS);					//select Display
					CLR(DISP_RS);					//clear RS line
					for (int a = 0;a<512;a+=2)
						{
							SPDR = Buffer[a+1];	// erste acht daten bits
							while(!(SPSR & (1<<SPIF)));
							SPDR = Buffer[a];		// zweiten acht bits
							while(!(SPSR & (1<<SPIF)));
						}
					SET(DISP_CS);
					}

			// Das letze Cluster wird ausgegeben.
					fat_read_file (Clustervar,Buffer,(cetemp));
					CLR(DISP_CS);					//select Display
					CLR(DISP_RS);					//clear RS line

					for (int a = 0;a<(pixeltemp);a+=2)
						{
							SPDR = Buffer[a+1];	// erste acht daten bits
							while(!(SPSR & (1<<SPIF)));
							SPDR = Buffer[a];		// zweiten acht bits
							while(!(SPSR & (1<<SPIF)));
						}
				SET(DISP_CS);
			}


// --------------------------------------------------------
			// Es sind 8 Bit Farbtiefe
			else if (farbtiefe == 8)   //8 bit Farbtiefe
			{
				while (Offsetbmp > 512)
				{
					Offsetbmp = Offsetbmp - 512;
					anfang ++;
				}

				fat_read_file (Clustervar,Buffer,anfang);

				pixeltemp = (pixel - ((512 - Offsetbmp)));	
				for (int ce = 1; pixeltemp>512; ce++)
				{
					pixeltemp = pixeltemp - 512;
					cetemp = ce;
				}
				cetemp = cetemp + anfang;
				S65_8bit_data();	// Konfigueriert das Display im 8 Bit Farb modus

				//______________________________________________
				//Konfiguert den Speicherzugriff, und legt den Bild Bereich fest.
				S65_wrcmd16(0xEF90);		S65_wrcmd16(0x0504);
				DISP_WrCmd(0x08);		DISP_WrCmd(x);
				DISP_WrCmd(0x09);		DISP_WrCmd(x + bmpHeight - 1);
				DISP_WrCmd(0x0A);		DISP_WrCmd(y);
				DISP_WrCmd(0x0B);		DISP_WrCmd(y + bmpWidth - 1);
				//___________________________________________________
				
				CLR(DISP_CS);					//select Display
				CLR(DISP_RS);					//clear RS line
				for (int a = Offsetbmp;a<512;a++)
					{
						SPDR = Buffer[a];		// acht bits
						while(!(SPSR & (1<<SPIF)));
					}
				SET(DISP_CS);
				for (int b = anfang + 1;b<(cetemp+1);b++)
					{
					fat_read_file (Clustervar,Buffer,b);
					CLR(DISP_CS);					//select Display
					CLR(DISP_RS);					//clear RS line

					for (int a = 0;a<512;a++)
						{
							SPDR = Buffer[a];		// acht bits
							while(!(SPSR & (1<<SPIF)));
						}
					SET(DISP_CS);
					}

			// Das letze Cluster wird ausgegeben.
					fat_read_file (Clustervar,Buffer,(cetemp+1));
					CLR(DISP_CS);					//select Display
					CLR(DISP_RS);					//clear RS line

					for (int a = 0;a<(pixeltemp);a++)
						{
							SPDR = Buffer[a];		// acht bits
							while(!(SPSR & (1<<SPIF)));
						}
				SET(DISP_CS);
				S65_16bit_data();

				return(0);
			}
			else	// Wenn Farbtiefe nicht 16 Bit und nicht 8 Bit, liegt ein fehler vor.
			{
				return(1);
			}
		}
return(1);		// Alles klar, keine Fehler
//Load_BMP_Ende	
}


void S65_16bit_data()
{
	S65_wrcmd16(0xEF90);
	S65_wrcmd16(0xE80F);
	S65_refresh();
}

void S65_8bit_data()
{
	S65_wrcmd16(0xEF90);
	S65_wrcmd16(0xE800);
	//S65_wrcmd16(0xE801);
	S65_refresh();
}

void S65_refresh()
{
	S65_wrcmd16(0xEF90);
	S65_wrcmd16(0x0000);
}
